//
//  t800_alu4.v
//   Arithmetic logical unit for 4bit
//
//  Copyright (C) 2020 Takayuki Hara
//  All rights reserved.
//                               http://hraroom.s602.xrea.com/msx/ocm/index.html
//
//  {\tgEFAі{\tgEFAɊÂč쐬ꂽh́Aȉ̏
//  ꍇɌAĔЕzюgp܂B
//
//  1.\[XR[h`ōĔЕzꍇAL̒쌠\A{ꗗAщL
//    Ɛӏ̂܂܂̌`ŕێ邱ƁB
//  2.oCi`ōĔЕzꍇAЕzɕt̃hLg̎ɁAL
//    쌠\A{ꗗAщLƐӏ܂߂邱ƁB
//  3.ʂɂ鎖ŐȂɁA{\tgEFA̔AяƓIȐi⊈
//    ɎgpȂƁB
//
//  {\tgEFÁA쌠҂ɂāû܂܁v񋟂Ă܂B쌠҂́A
//  ړIւ̓K̕ۏ؁Ai̕ۏ؁A܂Ɍ肳ȂAȂ閾
//  I͈ÖقȕۏؐӔC܂B쌠҂́AR̂킸AQ
//  ̌킸AӔC̍_ł邩iӔCł邩iߎ
//  ̑́js@sׂł邩킸Aɂ̂悤ȑQ\m
//  ĂƂĂA{\tgEFA̎gpɂĔi֕i܂͑pT
//  [rX̒BAgp̑rAf[^̑rAv̑rAƖ̒f܂߁A܂
//  Ɍ肳ȂjڑQAԐڑQAIȑQAʑQAIQA
//  ͌ʑQɂāAؐӔC𕉂Ȃ̂Ƃ܂B
//
//  Note that above Japanese version license is the formal document.
//  The following translation is only for reference.
//
//  Redistribution and use of this software or any derivative works,
//  are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials
//     provided with the distribution.
//  3. Redistributions may not be sold, nor may they be used in a
//     commercial product or activity without specific prior written
//     permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
//  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
//  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
//  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
//  POSSIBILITY OF SUCH DAMAGE.

module t800_alu4 (
	input	[ 7:0]	opecode,
	input	[ 3:0]	acc_q,
	input	[ 3:0]	operand_q,
	input			cy_q,
	output	[ 3:0]	result_d,
	output			cy_d
);
	wire	[ 4:0]	w_add;
	wire	[ 4:0]	w_sub;
	wire	[ 3:0]	w_and;
	wire	[ 3:0]	w_xor;
	wire	[ 3:0]	w_or;
	wire	[ 4:0]	w_result;

	assign w_add	= { 1'b0, acc_q } + { 1'b0, operand_q } + { 4'd0, cy_q };
	assign w_sub	= { 1'b0, acc_q } - { 1'b0, operand_q } - { 4'd0, cy_q };
	assign w_and	= acc_q & operand_q;
	assign w_xor	= acc_q ^ operand_q;
	assign w_or		= acc_q | operand_q;

	function [ 4:0] mux;
		input	[7:0]	opecode;
		input	[4:0]	w_add;
		input	[4:0]	w_sub;
		input	[3:0]	w_and;
		input	[3:0]	w_xor;
		input	[3:0]	w_or;

		casex( opecode )
		8'b1x00xxxx, 8'b00xxx100:
			mux = w_add;
		8'b1x01xxxx, 8'b1x111xxx, 8'b00xxx101:
			mux = w_sub;
		8'b1x100xxx:
			mux = { 1'b0, w_and };
		8'b1x101xxx:
			mux = { 1'b0, w_xor };
		8'b1x110xxx:
			mux = { 1'b0, w_or };
		default:
			mux = 5'd0;
		endcase
	endfunction

	assign w_result	= mux( opecode, w_add, w_sub, w_and, w_xor, w_or );

	assign result_d	= w_result[3:0];
	assign cy_d		= w_result[4];
endmodule
